/*

	(c) Shopping-ManaJer develoment-team
	This Code is published unter the Gnu-GPL v2

	Modified for JippySnake7
*/

import java.awt.Canvas;
import java.awt.Color;
import javax.swing.JLabel;

class GUIUpdater {
	private Renderer renderer;
	private Renderer lcRenderer;
	private JSLevel level;
	private JLabel lHud;
	private Snake snk;
	private Player player;
	private int type;
	private boolean reachedUltraSnake;
	private boolean drawPaused;
	private int[] playerLevelDone;
	
	public GUIUpdater(boolean hd, Canvas c, Canvas lc, int t, JLabel lh, JSLevel l, Snake s, Player p) {
		level = l;
		lHud = lh;
		snk = s;
		type = t;
		player = p;
		reachedUltraSnake = false;
		drawPaused = false;
		playerLevelDone = new int[100];
		for (int i = 0; i < 100; i++) playerLevelDone[i] = -1;
		if (hd)
			renderer = new Renderer(c, "res/textures-HD/textures", l.getNumLayers());
		else
			renderer = new Renderer(c, "res/textures/textures", l.getNumLayers());

		lcRenderer = new Renderer(lc, "res/gui/textures", 0);
	}
	
	public synchronized void draw(boolean paused, boolean makeBlind, Coord crd) {
		if (paused && drawPaused) {
			lcRenderer.drawScreen();
			renderer.flushScreen();
			return;
		}
		if (!paused && drawPaused) drawPaused = false;
		//Update the HUD
		String newHudText = "<HTML>";
		if (level.getNumApples() >= 250) {
			newHudText = newHudText.concat("ULTRASNAKE!!!   ");
			if (!reachedUltraSnake) {
				reachedUltraSnake = true;
				player.incrUltraSnakes();
				player.savePlayer();
			}
		}
		newHudText = newHudText.concat("&Auml;pfel: "+level.getNumApples()+"/"+level.getMinApples());
		if (level.getType() != 1) {
			newHudText = newHudText.concat("|| L&auml;ngste Schlange: "+player.getLongestSnake(level.getType(), level.getLevelNum()));
		}
		
		newHudText = newHudText.concat("<br>&Auml;pfel bis neue: ");
		if (level.getType()>2)
			newHudText = newHudText.concat(""+level.applesTillNew()+"<br>");
		else
			newHudText = newHudText.concat("---<br>");
		
		newHudText = newHudText.concat("Verbleibene Zeit: ");
		if (level.timerUsed())
			newHudText = newHudText.concat(""+formatInSec(level.remainingTime())+"s");
		else
			newHudText = newHudText.concat("---");
		lHud.setText(newHudText+"</HTML>");
		
		//renew all snake-layers:
		for (int i = 0; i < level.getNumLayers(); i++)
			renderer.reNewLayer(2*i+1);
		//drawSnake
		drawSnake();
		
		//finallay: render everything
		renderer.preRender();
		if (makeBlind) {
			renderer.clearScreen(Color.BLACK, Coord.getZeroCoord(), new Coord(20,crd.getY()-5),crd.getZ());
			renderer.clearScreen(Color.BLACK, new Coord(0,crd.getY()+5), new Coord(20,20),crd.getZ());
			renderer.clearScreen(Color.BLACK, Coord.getZeroCoord(), new Coord(crd.getX()-5,20),crd.getZ());
			renderer.clearScreen(Color.BLACK, new Coord(crd.getX()+5,0), new Coord(20,20),crd.getZ());
			renderer.drawTextureOnScreen(1100, crd.add(new Coord(-4,-4)), new Coord(-1*crd.getZ(), -1*crd.getZ()));
		}
		if (paused) {
			renderer.drawTextureOnScreen(1101, Coord.getZeroCoord(), Coord.getZeroCoord());
			drawPaused = true;
		}
		renderer.flushScreen();
		drawLevelChooser();
	}
	
	public void resetReachedUltraSnake() {
		reachedUltraSnake = false;
	}
	
	//draws the level
	public void drawLevel() {
		renderer.reNewAllLayers();
		for (Coord crd : Coord.range(Coord.getZeroCoord(), level.getSize().add(new Coord(-1,-1,-1)))) {
			renderer.drawTexture(0,level.getLayerDown(crd),crd);
			renderer.drawTexture(2*(crd.getZ()+1),level.getLayer(crd),crd);
			renderer.drawTexture(2*level.getNumLayers()+1,level.getLayerTop(crd),crd);
		}
	}
	
	private void drawSnake() {
		for (Coord crd : Coord.range(Coord.getZeroCoord(), level.getSize().add(new Coord(-1,-1,-1))))
			if (level.isAppleAt(crd)) renderer.drawTexture(2*crd.getZ()+1,999,crd);
		renderer.drawTexture(2*snk.getHeadPos().getZ()+1,snk.getHeadTexture(), snk.getHeadPos());
		for (SnakeElement s: snk.getBodyElements())
			renderer.drawTexture(2*s.getPos().getZ()+1,s.getTexture(),s.getPos());
	}
	
	public void drawLevelChooser() {
		lcRenderer.clearScreen(new Color(192,192,192), Coord.getZeroCoord(), new Coord(9,9),0);
		for (int r = 0; r < Globals.NUM_LEVELS[type]/10; r++) {
			for (int i = 0; i < 10; i++) {
				int n = 10*r+i;
				int d = player.getLevelDone(type, n);
				if (playerLevelDone[n] != d) {
					playerLevelDone[n] = d;
					lcRenderer.drawTexture(0, d,new Coord(i,r));
				}
			}
		}
		lcRenderer.drawScreen();
	}
	
	public void showGame() {
		renderer.flushScreen();
	}
	
	public void showLevelChooser() {
		lcRenderer.flushScreen();
	}
	
	private String formatInSec(int time) {
		//return time as a string in format xx.yy.
		// time is representet in quarterseconds: 1s=time <=> 4=time
		int fracSec = time%4;
		int sec = (int) (time/4);
		String timeStr = sec+".";
		fracSec *= 25;
		if (fracSec == 0) timeStr = timeStr.concat("00");
		else if (fracSec == 50) timeStr = timeStr.concat("50");
		else timeStr = timeStr.concat(Integer.toString(fracSec));
		return timeStr;
	}
}