/*(c) JippySnake7 Developmentteam.
 *
 *
 * This code is published unter GNU Public Licence v2.
 *
 *
 */
import java.io.*;
import java.awt.event.KeyEvent;
class Player {
	private int[][] levelsDone;
	//0: not playable
	//1: playable
	//2: done
	//3: filled
	
	private int[][] longestSnake;
	
	private int numUltraSnakes;
	private String file;
	
	//Options
	private int keyUp;
	private int keyDown;
	private int keyLeft;
	private int keyRight;
	private int keyPause;
	
	private String lang;
	private boolean playSounds;
	
	public Player() {
		levelsDone = new int[Globals.NUM_LEVELS.length][];
		levelsDone[0] = new int[Globals.NUM_LEVELS[0]];
		levelsDone[1] = new int[Globals.NUM_LEVELS[1]];
		levelsDone[2] = new int[Globals.NUM_LEVELS[2]];
		levelsDone[3] = new int[Globals.NUM_LEVELS[3]];
		levelsDone[4] = new int[Globals.NUM_LEVELS[4]];
		longestSnake = new int[Globals.NUM_LEVELS.length][];
		longestSnake[0] = new int[Globals.NUM_LEVELS[0]];
		longestSnake[1] = new int[Globals.NUM_LEVELS[1]];
		longestSnake[2] = new int[Globals.NUM_LEVELS[2]];
		longestSnake[3] = new int[Globals.NUM_LEVELS[3]];
		longestSnake[4] = new int[Globals.NUM_LEVELS[4]];
		keyUp = KeyEvent.VK_UP;
		keyDown = KeyEvent.VK_DOWN;
		keyLeft = KeyEvent.VK_LEFT;
		keyRight = KeyEvent.VK_RIGHT;
		keyPause = KeyEvent.VK_NUMPAD0;
		lang = "de";
		playSounds = true;
	}
	
	public static Player loadPlayer(String file) {
		Player p = new Player();
		p.file = file;
		int type;
		try {
			BufferedReader buffR = new BufferedReader(new FileReader(file));
			String line = buffR.readLine();
			String[] parts = line.split("::");
			while (!line.equals("@@@")) {
				if (parts[0].equals("ULTRASNAKES")) {
					p.numUltraSnakes = Integer.parseInt(parts[1]);
				} else if (parts[0].equals("LEVEL_DONE")) {
					type = Integer.parseInt(parts[1]);
					p.levelsDone[type] = new int[Globals.NUM_LEVELS[type]];
					for (int i = 0; i < Globals.NUM_LEVELS[type]/10; i++) {
						String li = buffR.readLine();
						String[] l = li.split(" ");
						for(int j = 0; j < 10; j++)
							p.levelsDone[type][10*i+j]=Integer.parseInt(l[j]);
					}
				} else if (parts[0].equals("LONGEST_SNAKE")) {
					type = Integer.parseInt(parts[1]);
					p.longestSnake[type] = new int[Globals.NUM_LEVELS[type]];
					for (int i = 0; i < Globals.NUM_LEVELS[type]/10; i++) {
						String li = buffR.readLine();
						String[] l = li.split(" ");
						for(int j = 0; j < 10; j++)
							p.longestSnake[type][10*i+j]=Integer.parseInt(l[j]);
					}
				} else
					System.out.println("Unkown line: "+line);
				line = buffR.readLine();
				parts = line.split("::");
			}
		} catch (FileNotFoundException e) {
			p.levelsDone[0][0] = 1;
			p.levelsDone[0][1] = 1;
			p.levelsDone[0][2] = 1;
			p.levelsDone[0][3] = 1;
			p.levelsDone[1][0] = 1;
			p.levelsDone[2][0] = 1;
			p.levelsDone[2][1] = 1;
			p.levelsDone[2][2] = 1;
			p.levelsDone[2][3] = 1;
			p.levelsDone[3][0] = 1;
			p.levelsDone[3][1] = 1;
			p.levelsDone[3][2] = 1;
			p.levelsDone[3][3] = 1;
			p.levelsDone[4][0] = 1;
			p.levelsDone[4][1] = 1;
			p.levelsDone[4][2] = 1;
			p.levelsDone[4][3] = 1;
			p.savePlayer();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return p;
	}
	
	public void savePlayer() {
		try {
			BufferedWriter buffW = new BufferedWriter(new FileWriter(file));
			String tmp;
			tmp = "ULTRASNAKES::"+Integer.toString(numUltraSnakes);
			buffW.write(tmp,0,tmp.length());
			buffW.newLine();
			for (int i = 0; i < Globals.NUM_LEVELS.length; i++) {
				tmp = "LEVEL_DONE::"+Integer.toString(i);
				buffW.write(tmp,0,tmp.length());
				buffW.newLine();
				for (int r = 0; r < Globals.NUM_LEVELS[i]/10; r++) {
					tmp = "";
					for (int j = 0; j < 10; j++)
						tmp = tmp.concat(Integer.toString(levelsDone[i][10*r+j])+" ");
					buffW.write(tmp,0,tmp.length());
					buffW.newLine();
				}
			}
			for (int i = 0; i < Globals.NUM_LEVELS.length; i++) {
				if (i != 1) {
					tmp = "LONGEST_SNAKE::"+Integer.toString(i);
					buffW.write(tmp,0,tmp.length());
					buffW.newLine();
					for (int r = 0; r < Globals.NUM_LEVELS[i]/10; r++) {
						tmp = "";
						for (int j = 0; j < 10; j++)
							tmp = tmp.concat(Integer.toString(longestSnake[i][10*r+j])+" ");
						buffW.write(tmp,0,tmp.length());
						buffW.newLine();
					}
				}
			}
			buffW.write("@@@",0,3);
			buffW.newLine();
			buffW.flush();
		} catch (Exception e) {
			System.out.println("bla");
		}
	}
	
	public void incrUltraSnakes() {
		numUltraSnakes++;
	}
	
	public void setLevelDone(int t, int n, int v) {
		levelsDone[t][n] = v;
	}
	
	public void setLongestSnake(int t, int n, int l) {
		longestSnake[t][n] = Math.max(longestSnake[t][n], l);
	}
	public int getLongestSnake(int t, int n) {
		return longestSnake[t][n];
	}
	
	public void unLockLevel(int t) {
		int n = 0;
		while ((n < Globals.NUM_LEVELS[t]) && (levelsDone[t][n] > 0)) n++;
		if (n < Globals.NUM_LEVELS[t]) setLevelDone(t, n, 1);
	}
	
	public int getNumUltraSnakes() {
		return numUltraSnakes;
	}
	
	public int getLevelDone(int t, int n) {
		return levelsDone[t][n];
	}
	
	
}