/*(c) JippySnake Entwicklungsteam.
 *
 *
 *Dieser Code wird unter der GNU Public Licence vertireben
 *
 *
 */

/* Modified for Shopping-ManaJer */

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.BufferStrategy;
import java.util.HashMap;
import java.net.URL;
import java.net.URI;
import javax.imageio.ImageIO;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Font;
import java.io.*;
import java.awt.RenderingHints;
import java.awt.RenderingHints.Key;

class Renderer {
	private Graphics2D canvasGr;
	private BufferedImage canvasImg;
	private BufferStrategy buffStrat;
	private Canvas canvas;
	private HashMap<RenderingHints.Key, Object> rendHints;
	private BufferedImage[] buffImg;
	private Graphics2D[] buffImgGr;
	public BufferedImage[] texture;
	
	private String gameRoot;
	private int numLayers;
	private int textSize;
	private int offSize;
	
	public Renderer(Canvas c, String textFile, int numLayers) {
		//We need a buttom, a top an 2 layers for each levelLayer
		/* Strukture: N means the number of layers
		                              layer
		               TopLayer        2N+1
		               BlockLayer N    2N
		               SnakeLayer N    2N-1
		                    .
		                    .
		                    .
		               BlockLayer 0    2
		               SnakeLayer 0    1
		               ButtomLayer     0
		*/
		buffImg = new BufferedImage[2+2*numLayers];
		buffImgGr = new Graphics2D[2+2*numLayers];
		this.numLayers = 2+2*numLayers;
		rendHints = new HashMap<RenderingHints.Key, Object>();
		rendHints.put(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_SPEED);
		gameRoot = Globals.GAMEROOT;
		textSize = 5;
		loadTextures(textFile);
		initRenderer(c);
		offSize = textSize/5;
	}
	
	private void initRenderer(Canvas c) {
		//init Graphics interface to canvas
		canvas = c;
		canvasImg = new BufferedImage(20*textSize,20*textSize,BufferedImage.TYPE_INT_ARGB);
		canvasGr = canvasImg.createGraphics();
		canvasGr.addRenderingHints(rendHints);
		
		//init offscreen layers
		for (int i = 0; i < buffImg.length; i++) {
			buffImg[i] = new BufferedImage(20*textSize,20*textSize,BufferedImage.TYPE_INT_ARGB);
			buffImgGr[i] = buffImg[i].createGraphics();
		}
		canvasGr.setFont(new Font("",Font.PLAIN,10));
		canvasGr.drawString("",0,0);
		
	}
	
	public void drawText(String text, Color color, Coord crd, int yOffset) {
		text = text.replaceAll("&Uuml;", "Ü");
		text = text.replaceAll("&uuml;", "ü");
		text = text.replaceAll("&Ouml;", "Ö");
		text = text.replaceAll("&ouml;", "ö");
		text = text.replaceAll("&Auml;", "A");
		text = text.replaceAll("&auml;", "ä");
		
		canvasGr.setColor(color);
		canvasGr.drawString(text, crd.getX()*15,crd.getY()*textSize+textSize+yOffset);
	}
	
	public void drawTexture(int layer, int texNr, Coord crd) {
		drawImage(layer, texNr, crd, -offSize*((int) layer/2),-offSize*((int) layer/2));
	}

	public void drawTextureOnScreen(int texNr, Coord crd, Coord offset) {
		drawImageOnScreen(texNr, crd, offSize*offset.getX(), offSize*offset.getY());
	}
	
	public void clearScreen(Color col, Coord pos1, Coord pos2, int offset) {
		canvasGr.setColor(col);
		canvasGr.fillRect(pos1.getX()*textSize-offSize*offset,pos1.getY()*textSize-offSize*offset,pos2.sub(pos1).getX()*textSize+textSize,pos2.sub(pos1).getY()*textSize+textSize);
	}
	
	public void clearRect(int layer, Color col, Coord pos, int yOffset) {
		buffImgGr[layer].setColor(col);
		buffImgGr[layer].fillRect(pos.getX()*textSize,pos.getY()*textSize+yOffset, textSize, textSize);
		
	}
	
	public void reNewLayer(int layer) {
		buffImg[layer] = new BufferedImage(20*textSize,20*textSize,BufferedImage.TYPE_INT_ARGB);
		buffImgGr[layer] = buffImg[layer].createGraphics();
	}
	
	public void reNewAllLayers() {
		for (int layer = 0; layer < buffImg.length; layer++) {
			buffImg[layer] = new BufferedImage(20*textSize,20*textSize,BufferedImage.TYPE_INT_ARGB);
			buffImgGr[layer] = buffImg[layer].createGraphics();
		}
	}
	
	public synchronized void drawScreen() {
		preRender();
		flushScreen();
	}
	
	public void flushScreen() {
		try {
			Graphics g = canvas.getGraphics();
			g.drawImage(canvasImg,0,0,canvas);
		} catch (Exception e) {
		}
	}
	
	public void preRender() {
		for (BufferedImage b : buffImg)
			canvasGr.drawImage(b, new AffineTransform(), canvas);
	}
	
	public void drawImageOnScreen(int txt, Coord pos, int xOffset, int yOffset) {
		try {
			AffineTransform a = new AffineTransform(1.0, 0.0, 0.0, 1.0, pos.getX()*textSize*1.0+xOffset, pos.getY()*textSize*1.0+yOffset);
			canvasGr.drawImage(texture[txt], a, canvas);
		} catch (Exception e) {
			System.out.println("Error: "+e.getMessage());
		}
	}
	
	public void drawImage(int layer, int txt, Coord pos, int xOffset, int yOffset) {
		AffineTransform a = new AffineTransform(1.0, 0.0, 0.0, 1.0, pos.getX()*textSize*1.0+xOffset, pos.getY()*textSize*1.0+yOffset);
		try {
			buffImgGr[layer].drawImage(texture[txt], a, canvas);
		} catch (IndexOutOfBoundsException e) {
		} catch (Exception e) {
			System.out.println("Error: "+e.getMessage());
		}
	}
	
	private void loadTextures(String file) {
		URI texFile;
		URL url;
		BufferedImage img;
		FileReader fileReader;
		String line;
		String[] parts;
		try {
			texFile = new URI(new URL(Globals.GAMEROOT+file).toString());
			fileReader = new FileReader(new File(texFile));
			BufferedReader buffReader = new BufferedReader(fileReader);
			texture = new BufferedImage[2000];
			line = buffReader.readLine();
			while (!line.equals("@@@")) {
				parts = line.split("::");
				if (parts[0].equals("include"))
					loadTextures(parts[1]);
				else if (parts[0].equals("size"))
					textSize = Integer.parseInt(parts[1]);
				else {
					url = new URL(gameRoot+"res/"+parts[1]);
					img = ImageIO.read(url);
					texture[Integer.parseInt(parts[0])] = img;
				}
				line = buffReader.readLine();
			}
		} catch (FileNotFoundException e) {
			System.out.println("Texture-file not found:"+file);
			System.exit(0);
		} catch (Exception e) {
			System.out.println("Error while reading Textures.");
			e.printStackTrace();
			System.out.println(e.getMessage());
			System.exit(0);
		}
	}
}